import { Table } from './interface'
import { reactive, computed, onMounted, toRefs } from 'vue'
import { AxiosResponse } from 'axios'

interface resposeDataTypes extends AxiosResponse {
  code?: string
  totalSize?: number
}

/** table 页面操作方法封装 */
export const useTable = (
  /** 获取表格数据api方法 */
  api: (params: any) => Promise<resposeDataTypes>,
  /** 获取数据初始化参数 */
  initParam: object = {},
  /** 是否有分页
   * true: 有
   * false: 无
   */
  isPageable = true,
  /** 后台返回数据处理方法 */
  dataCallBack?: (data: any) => any,
) => {
  /** 基本参数数据 */
  const state = reactive<Table.TableStateProps>({
    loading: false,
    stopGet: false,
    tableData: [],
    pageable: {
      page: 1,
      pageSize: 20,
      totalNum: 0,
    },
    searchParam: {},
    searchInitParam: {},
    totalParam: {},
  })

  /** 分页查询参数 */
  const pageParam = computed({
    get: () => {
      return {
        page: state.pageable.page,
        pageSize: state.pageable.pageSize,
      }
    },
    set: (newVal: any) => {
      console.log(newVal)
    },
  })

  /** 初始化处理
   * 设置表单查询默认值
   * 获取表格数据
   */
  onMounted(() => {
    reset()
  })

  /** 获取表格数据 */
  async function getTableList() {
    try {
      console.log(123)
      state.loading = true
      Object.assign(state.totalParam, isPageable ? pageParam.value : {})
      console.log(state.totalParam)
      const resData = await api({ ...initParam, ...state.totalParam })
      console.log(resData)
      if (resData.code === '000000') {
        let data = resData.data
        dataCallBack && (data = dataCallBack(data))
        state.tableData = data
        isPageable && updatePageable(resData.totalSize || 0)
        console.log(state.tableData)
        state.loading = false
      }
      state.loading = false
    } catch (error) {
      state.loading = false
      console.log(error)
    }
  }

  /** 更新查询参数 */
  function updatedTotalParam() {
    state.totalParam = {}
    /** 处理查询参数，可以给查询参数增加自定义前缀操作 */
    const nowSearchParam: { [key: string]: any } = {}
    /** 防止手动清空输入框携带参数（这里可以自定义查询参数前缀） */
    for (const key in state.searchParam) {
      /** 某些情况下参数为 false/0 也应该携带参数 */
      if (
        state.searchParam[key] ||
        state.searchParam[key] === false ||
        state.searchParam[key] === 0
      ) {
        nowSearchParam[key] = state.searchParam[key]
      }
    }
    Object.assign(
      state.totalParam,
      nowSearchParam,
      isPageable ? pageParam.value : {},
    )
  }

  /** 更新总数据量信息 */
  function updatePageable(total: number) {
    Object.assign(state.pageable, { totalNum: total })
  }

  /** 表格数据查询 */
  function search() {
    state.pageable.page = 1
    updatedTotalParam()
    getTableList()
  }

  /** 表格数据重置 */
  function reset() {
    state.pageable.page = 1
    state.searchParam = {}
    /** 重置搜索表单的时，如果有默认搜索参数，则重置默认的搜索参数 */
    Object.keys(state.searchInitParam).forEach((key) => {
      state.searchParam[key] = state.searchInitParam[key]
    })
    updatedTotalParam()
    getTableList()
  }

  /** 每页条数变化处理 */
  function handleSizeChange(
    /** 当前页 */
    current: number,
    /** 每页条数 */
    size: number,
  ) {
    if (state.stopGet) {
      state.stopGet = false
      return
    }
    if (state.pageable.page !== current) {
      state.pageable.page = current
    } else if (state.pageable.pageSize !== size) {
      state.pageable.page = 1
      state.pageable.pageSize = size
      state.stopGet = true
    } else {
      return
    }

    getTableList()
  }

  /** 当前页变化 */
  function handleCurrentChange(
    /** 当前页 */
    current: number,
    /** 每页条数 */
    // size?: number
  ) {
    state.pageable.page = current
    getTableList()
  }

  /** loading变化 */
  function changeLoading(val: boolean) {
    state.loading = val
  }
  return {
    ...toRefs(state),
    /** 获取表格数据 */
    getTableList,
    /** 表格数据查询 */
    search,
    /** 表格数据重置 */
    reset,
    /** 每页条数变化处理 */
    handleSizeChange,
    /** 当前页变化 */
    handleCurrentChange,
    /** loading变化处理 */
    changeLoading,
  }
}
